@@ -7,12 +7,12 @@ package IO::Async::Connector;
 
 use strict;
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 use IO::Async::Notifier;
 
 use POSIX qw( EINPROGRESS );
-use Socket qw( SO_ERROR );
+use Socket qw( SOL_SOCKET SO_ERROR );
 
 use Carp;
 
@@ -99,11 +99,10 @@ sub _get_sock_err
 {
    my ( $sock ) = @_;
 
-   my $err_packed = $sock->sockopt( SO_ERROR );
-
-   if( defined $err_packed ) {
-      my $err = unpack( "I", $err_packed );
+   my $err = $sock->getsockopt( SOL_SOCKET, SO_ERROR );
 
+   if( defined $err ) {
+      # 0 means no error, but is still defined
       return undef if !$err;
 
       $! = $err;
